@@ -20,9 +20,9 @@ import socket
 from unittest import mock
 
 import netaddr
-from oslotest import base as test_base
 
 from oslo_utils import netutils
+from oslo_utils.tests import base as test_base
 
 
 class NetworkUtilsTest(test_base.BaseTestCase):
@@ -404,8 +404,8 @@ class NetworkUtilsTest(test_base.BaseTestCase):
             """Iface	Destination	Gateway 	Flags	RefCnt	Use	Metric	Mask		MTU	Window	IRTT
 eth0	00000000	01cc12ac	0003	0	0	600	00000000	0	0	0
 eth0	00cc12ac	00000000	0001	0	0	600	00FFFFFF	0	0	0
-eth1	00cd12ac	00000000	0001	0	0	600	00FFFFFF	0	0	0"""
-        )  # noqa : E501
+eth1	00cd12ac	00000000	0001	0	0	600	00FFFFFF	0	0	0"""  # noqa : E501
+        )
 
         addr = namedtuple('addr', ['family', 'address'])
         mock_ifaddrs.return_value = {
@@ -429,8 +429,8 @@ eth1	00cd12ac	00000000	0001	0	0	600	00FFFFFF	0	0	0"""
         mock_open.return_value = io.StringIO(
             """00000000000000000000000000000000 00 00000000000000000000000000000000 00 20010db8000000000000000000000001 00000000 00000000 00000000 08000000 eth0
 20010db8000000000000000000000000 31 00000000000000000000000000000000 00 00000000000000000000000000000000 00000000 00000000 00000000 08000000 eth0
-20010db8100000000000000000000000 31 00000000000000000000000000000000 00 00000000000000000000000000000000 00000000 00000000 00000000 08000000 eth1"""
-        )  # noqa: E501
+20010db8100000000000000000000000 31 00000000000000000000000000000000 00 00000000000000000000000000000000 00000000 00000000 00000000 08000000 eth1"""  # noqa: E501
+        )
 
         addr = namedtuple('addr', ['family', 'address'])
         mock_ifaddrs.return_value = {
@@ -454,8 +454,8 @@ eth1	00cd12ac	00000000	0001	0	0	600	00FFFFFF	0	0	0"""
         mock_open.return_value = io.StringIO(
             """Iface	Destination	Gateway 	Flags	RefCnt	Use	Metric	Mask		MTU	Window	IRTT
 eth0	00cc12ac	00000000	0001	0	0	600	00FFFFFF	0	0	0
-eth1	00cd12ac	00000000	0001	0	0	600	00FFFFFF	0	0	0"""
-        )  # noqa : E501
+eth1	00cd12ac	00000000	0001	0	0	600	00FFFFFF	0	0	0"""  # noqa : E501
+        )
 
         self.assertEqual('127.0.0.1', netutils._get_my_ipv4_address())
         mock_open.assert_called_once_with('/proc/net/route')
@@ -468,8 +468,8 @@ eth1	00cd12ac	00000000	0001	0	0	600	00FFFFFF	0	0	0"""
     ):
         mock_open.return_value = io.StringIO(
             """20010db8000000000000000000000000 31 00000000000000000000000000000000 00 00000000000000000000000000000000 00000000 00000000 00000000 08000000 eth0
-20010db8100000000000000000000000 31 00000000000000000000000000000000 00 00000000000000000000000000000000 00000000 00000000 00000000 08000000 eth1"""
-        )  # noqa: E501
+20010db8100000000000000000000000 31 00000000000000000000000000000000 00 00000000000000000000000000000000 00000000 00000000 00000000 08000000 eth1"""  # noqa: E501
+        )
 
         self.assertEqual('::1', netutils._get_my_ipv6_address())
         mock_open.assert_called_once_with('/proc/net/ipv6_route')
